-- Description:	Somme usefull functions
-- Author:		Damien "doug" gavard
-- Email:		damien.gavard@gmail.com
-- Site:		www.3rd-wing.net
-- Date:		March 2007
-- Lo version:	1.12a
-- Copyright (C) 2007 - Damien Gavard

-- *********************************************************************
-- *
-- * Convert {x,z} to Lat/Long
-- *
-- *********************************************************************

--data for recalculation
zeroX = 5000000
zeroZ = 6600000

centerX = 11465000 - zeroX --circle center
centerZ =  6500000 - zeroZ

pnSxW_X = 4468608 - zeroX -- point 40dgN : 24dgE
pnSxW_Z = 5730893 - zeroZ

pnNxW_X = 5357858 - zeroX -- point 48dgN : 24dgE
pnNxW_Z = 5828649 - zeroZ

pnSxE_X = 4468608 - zeroX -- point 40dgN : 42dgE
pnSxE_Z = 7269106 - zeroZ

pnNxE_X = 5357858 - zeroX -- point 48dgN : 42dgE
pnNxE_Z = 7171350 - zeroZ

lenNorth = math.sqrt((pnNxW_X-centerX)*(pnNxW_X-centerX) + (pnNxW_Z-centerZ)*(pnNxW_Z-centerZ))
lenSouth = math.sqrt((pnSxW_X-centerX)*(pnSxW_X-centerX) + (pnSxW_Z-centerZ)*(pnSxW_Z-centerZ))
lenN_S = lenSouth - lenNorth

RealAngleMaxLongitude = math.atan ((pnSxW_Z - centerZ)/(pnSxW_X - centerX)) * 180/math.pi
-- borders
EndWest = 24
EndEast = 42
EndNorth = 48
EndSouth = 40
MiddleLongitude = (EndWest + EndEast) / 2
ToLengthN_S = ((EndNorth - EndSouth) / lenN_S)
ToAngleW_E = (MiddleLongitude - EndWest) / RealAngleMaxLongitude

ToDegree = 360/(2*math.pi)


function getLongitude(x, z)	-- degrees , (x (meters) - to North, z (meters) - to East)

	ang = - math.atan (((z - centerZ)) / (x - centerX)) * ToDegree
	return ang * ToAngleW_E + MiddleLongitude

end

function getLatitude(x, z)	--degrees (x (meters) - to North, z (meters) - to East)

	len = lenSouth - math.sqrt((x-centerX)*(x-centerX) + (z-centerZ)*(z-centerZ))
	return len * ToLengthN_S + EndSouth

end

function getXYCoords(inLatitudeDegrees, inLongitudeDegrees) -- args: 2 numbers // Return two value in order: X, Y

	-- Lo coordinates system
	realAng = (inLongitudeDegrees - MiddleLongitude) / ToAngleW_E / ToDegree
	realLen = lenSouth - (inLatitudeDegrees - EndSouth) / ToLengthN_S
	outX = centerX - realLen * cos (realAng)
	outZ = centerZ + realLen * sin (realAng)
	return outX, outY
end



-- *********************************************************************
-- *
-- * Indique si les 3 positions forment des points d'atterrissage
-- *
-- *********************************************************************

function isLandingWpts(lat1, long1, lat2, long2, lat3, long3)
	local temp = false
	
	if lat1~=nil and lat2~=nil and lat3~=nil and long1~=nil and long2~=nil and long3~=nil then
	
		--Conversions
		local lat1Rad = lat1 * ((2 * math.pi) / 360)
		local long1Rad = long1 * ((2 *math.pi) / 360)
		
		local lat2Rad = lat2 * ((2 *math.pi) / 360)
		local long2Rad = long2 * ((2 * math.pi) / 360)
		
		local lat3Rad = lat3 * ((2 * math.pi) / 360)
		local long3Rad = long3 * ((2 *math.pi) / 360)
		
		
		--Le wpt2 est le FAF, on se rfre a lui
		
		--Calcul de la distance entre le FAF et le seuil de piste
		local deltaLat = lat1Rad - lat2Rad
		local deltaLong = long1Rad - long2Rad
		local a = math.sin(deltaLat/2) * math.sin(deltaLat/2) + math.cos(lat2Rad) * math.cos(lat1Rad) * math.sin(deltaLong/2) * math.sin(deltaLong/2)
		local c = 2 * math.atan2 (math.sqrt(a), math.sqrt(1-a) )
		local distancePafPiste = 6378.14 * c --En Km
		
		local deltaLat = lat3Rad - lat2Rad
		local deltaLong = long3Rad - long2Rad
		local a = math.sin(deltaLat/2) * math.sin(deltaLat/2) + math.cos(lat2Rad) * math.cos(lat3Rad) * math.sin(deltaLong/2) * math.sin(deltaLong/2)
		local c = 2 * math.atan2 (math.sqrt(a), math.sqrt(1-a) )
		local distancePafPai = 6378.14 * c --En Km
	
		if distancePafPiste<10 and distancePafPiste>7 and distancePafPai<10 and distancePafPai>7 then
			temp = true
		end
		
	end	

	return temp

end


-- *********************************************************************
-- *
-- * Enlve les caractres de ponctuation du nom
-- *
-- *********************************************************************

function washName(name)
    local temp = ""
    local i = 0
  	local j = 0
  	
	for tempParenthese in string.gfind(name, "[^(]+") do --On rcupre la partie juska la parenthese
		if i == 0 then --1re ( uniquement

			for tempEspace in string.gfind(tempParenthese, "[^ ]+") do --On rcupre la partie sans l'espace
				if j == 0 then
				    if tempEspace then
						temp = tempEspace
					else
					     temp = tempParenthese
					end
				end
			end

			j = j + 1
		end
		i = i + 1
	end
	return temp
end

-- *********************************************************************
-- *
-- * Enleve le (Me) dans la chaine de caractere
-- *
-- *********************************************************************
function delMe(input)
	local debut, fin = string.find(input, " [(]Me[)]")
	if debut == nil then
		return input
		--return 0
	else
		return string.sub(input, 0, debut) .. string.sub(input, fin+1, string.len(input))
		--return 1
	end
end

-- *********************************************************************
-- *
-- * Rcupre le type de l'avion (Ex:Su33 dans Su33 (Pilot1))
-- *
-- *********************************************************************
function getTypeName(input)
	local debut, fin = string.find(input, "[^(.+)]+")
	if debut ~= nil then --The type is valid
		if string.sub(input, debut, fin) == " " then
			fin = fin -1
		end
		local toReturn = string.sub(input, debut, fin)
		toReturn = string.gsub(toReturn, "[^A-Za-z0-9]", "")
		if toReturn == "1L13EWR" then
			toReturn = "EWR1L13"
		elseif toReturn == "2S6Tunguska" then
			toReturn = "Tunguska2S6"
		elseif toReturn == "55G6EWR" then
			toReturn = "EWR55G6"
		end
		return toReturn
	else --the type is empty
		return "Unknown"
	end
end

-- *********************************************************************
-- *
-- * Rcupre le nom de l'unit (Ex:Pilot1 dans Su33 (Pilot1))
-- *
-- *********************************************************************
function getUnitName(input)
	local debut, fin = string.find(input, "[(].*[)]")
	if debut==nil then
		return ""
	else
		return string.sub(input, debut+1, fin-1)
	end
end

function getGoodSignalType(input)
	if input=="scan" then
		return "Scan"
	elseif input=="lock" then
		return "Lock"
	elseif input=="missile_radio_guided" then
		return "MissileRadioGuided"
	elseif input=="track_while_scan" then
		return "TrackWhileScan"
	else
		return "Unknown"
	end	
end

function getGoodTrackingType(input)
	if input==2 then
		return "RadarView"
	elseif input==4 then
		return "EOSView"
	elseif input==8 then
		return "RadarLock"
	elseif input==10 then
		return "EOSLock"
	elseif input==20 then
		return "RadarTrack"
	elseif input==40 then
		return "EOSTrack"
	elseif input==200 then
		return "NetHumanPlane"
	elseif input==400 then
		return "AutoLockOn"
	elseif input==800 then
		return "LockOnJammer"
	else
		return "Unknown"
	end	
end

function writeObjectPropertyInStack(aStack, aObject, aObjectId)
	if aObject ~= nil and aObject.Country ~= "" then
		addString(aStack, "<id>" .. aObjectId .. "</id>")
		addString(aStack, "<commanderName>" .. getUnitName(delMe(aObject.Name)) .. "</commanderName>")
		addString(aStack, "<typeName>" .. getTypeName(aObject.Name) .. "</typeName>")
		addString(aStack, "<type>" .. getType(aObject.Type) .. "</type>")
		addString(aStack, "<subtype>" .. getSubType(aObject.Subtype) .. "</subtype>")
		addString(aStack, "<country>" .. aObject.Country .. "</country>")
		local strCoalition = ""
		if aObject.Coalition == "Enemies" then
			strCoalition = "Blue"
		elseif aObject.Coalition == "Allies" then
			strCoalition = "Red"
		else
			strCoalition = "Unknown"
		end
		addString(aStack, "<coalition>" .. strCoalition .. "</coalition>")
		addString(aStack, "<position>")
			addString(aStack, "<latitude>" .. aObject.LatLongAlt.Lat .. "</latitude>")
			addString(aStack, "<longitude>" .. aObject.LatLongAlt.Long .. "</longitude>")
			addString(aStack, "<altitude>" .. aObject.LatLongAlt.Alt .. "</altitude>")
		addString(aStack, "</position>")
		addString(aStack, "<heading>" .. aObject.Heading .. "</heading>")
	end
end

-- *********************************************************************
-- *
-- * Fonctions pour le StringBuilding de document XML
-- *
-- *********************************************************************
function newStack ()
	return {""}   -- starts with an empty string
end
    
function addString (stack, s)
	table.insert(stack, s)    -- push 's' into the the stack
	for i=table.getn(stack)-1, 1, -1 do
		if string.len(stack[i]) > string.len(stack[i+1]) then
			break
		end
		stack[i] = stack[i] .. table.remove(stack)
	end
end

function mydebug(data)
	local LogFile = io.open("./Temp/debug-lotatc2-debug.txt","w")
	if LogFile then
		LogFile:write(data .. "\n")
		LogFile:flush()
		io.close(LogFile)
	end
end

-- *********************************************************************
-- *
-- * Fonctions pour retourner un enum au lieu d'un int
-- *
-- *********************************************************************
function getType(input)
	if input==1 then
		return "Air"
	elseif input==2 then
		return "Ground"
	elseif input==3 then
		return "Navy"
	elseif input==4 then
		return "Weapon"
	elseif input==5 then
		return "Static"
	else
		return "Unknown"
	end	
end

function getSubType(input)
	if input==1 then
		return "Airplane"
	elseif input==2 then
		return "Helicopter"
	elseif input==8 then
		return "Moving"
	elseif input==9 then
		return "Standing"
	elseif input==17 then
		return "Tank"
	elseif input==16 then
		return "Sam"
	elseif input==12 then
		return "Sam"
	elseif input==4 then
		return "Missile"
	elseif input==5 then
		return "Bomb"
	elseif input==6 then
		return "Shell"
	elseif input==7 then
		return "Rocket"
	elseif input==13 then
		return "Airdrome"
	else
		return "Unknown"
	end	
end


